/* -*- c -*-
|| This file is part of Pike. For copyright information see COPYRIGHT.
|| Pike is distributed under GPL, LGPL and MPL. See the file COPYING
|| for more information.
*/

#include "global.h"

#include "config.h"

#include "stralloc.h"
#include "pike_macros.h"
#include "object.h"
#include "program.h"
#include "interpret.h"
#include "builtin_functions.h"
#include "pike_error.h"
#include "module_support.h"
#include "operators.h"

#define GLUT_API_VERSION 4

#ifdef HAVE_GLUT_GLUT_H
#include <GLUT/glut.h>
#else
#ifdef HAVE_GL_GLUT_H
#include <GL/glut.h>
#endif /* HAVE_GL_GLUT_H */
#endif /* HAVE_GLUT_GLUT_H */

static void pre_init(void)
{
}

static void f_glutInit(INT32 args)
{
  int argc = 1;
  char * ARGV[] = { "glut" };
  glutInit(&argc,ARGV);
  pop_n_elems(args);
}

static void post_init(void)
{
  add_function_constant("glutInit", f_glutInit,
			"function(void:void)", OPT_SIDE_EFFECT);
}




#ifndef MISSING_GLUTADDMENUENTRY
static void f_glutAddMenuEntry(INT32 args)
{
  char *arg1;
  INT32 arg2;

  check_all_args("glutAddMenuEntry", args, BIT_STRING, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.string->str;
  arg2=Pike_sp[1-args].u.integer;

  glutAddMenuEntry(arg1,arg2);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTADDMENUENTRY */

#ifndef MISSING_GLUTADDSUBMENU
static void f_glutAddSubMenu(INT32 args)
{
  char *arg1;
  INT32 arg2;

  check_all_args("glutAddSubMenu", args, BIT_STRING, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.string->str;
  arg2=Pike_sp[1-args].u.integer;

  glutAddSubMenu(arg1,arg2);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTADDSUBMENU */

#ifndef MISSING_GLUTATTACHMENU
static void f_glutAttachMenu(INT32 args)
{
  INT32 arg1;

  check_all_args("glutAttachMenu", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutAttachMenu(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTATTACHMENU */

#ifndef MISSING_GLUTBUTTONBOXFUNC
struct svalue glutButtonBoxFunc_fun;

void glutButtonBoxFunc_cb_wrapper(int arg1, int arg2)
{
  push_int(arg1);
  push_int(arg2);
  apply_svalue(&glutButtonBoxFunc_fun, 2);
  pop_stack();
}

static void f_glutButtonBoxFunc(INT32 args)
{
  if(TYPEOF(glutButtonBoxFunc_fun))
    free_svalue(&glutButtonBoxFunc_fun);
  assign_svalue_no_free(&glutButtonBoxFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutButtonBoxFunc", args, BIT_FUNCTION, 0);
  glutButtonBoxFunc((void *)&glutButtonBoxFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTBUTTONBOXFUNC */

#ifndef MISSING_GLUTCHANGETOMENUENTRY
static void f_glutChangeToMenuEntry(INT32 args)
{
  INT32 arg1;
  char *arg2;
  INT32 arg3;

  check_all_args("glutChangeToMenuEntry", args, BIT_INT, BIT_STRING, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.string->str;
  arg3=Pike_sp[2-args].u.integer;

  glutChangeToMenuEntry(arg1,arg2,arg3);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTCHANGETOMENUENTRY */

#ifndef MISSING_GLUTCHANGETOSUBMENU
static void f_glutChangeToSubMenu(INT32 args)
{
  INT32 arg1;
  char *arg2;
  INT32 arg3;

  check_all_args("glutChangeToSubMenu", args, BIT_INT, BIT_STRING, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.string->str;
  arg3=Pike_sp[2-args].u.integer;

  glutChangeToSubMenu(arg1,arg2,arg3);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTCHANGETOSUBMENU */

#ifndef MISSING_GLUTCOPYCOLORMAP
static void f_glutCopyColormap(INT32 args)
{
  INT32 arg1;

  check_all_args("glutCopyColormap", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutCopyColormap(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTCOPYCOLORMAP */

#ifndef MISSING_GLUTCREATEMENU
struct svalue glutCreateMenu_fun;

void glutCreateMenu_cb_wrapper(int arg1)
{
  push_int(arg1);
  apply_svalue(&glutCreateMenu_fun, 1);
  pop_stack();
}

static void f_glutCreateMenu(INT32 args)
{
  INT32 res;
  if(TYPEOF(glutCreateMenu_fun))
    free_svalue(&glutCreateMenu_fun);
  assign_svalue_no_free(&glutCreateMenu_fun, &Pike_sp[-args+0]);

  check_all_args("glutCreateMenu", args, BIT_FUNCTION, 0);
  res=glutCreateMenu((void *)&glutCreateMenu_cb_wrapper);
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTCREATEMENU */

#ifndef MISSING_GLUTCREATESUBWINDOW
static void f_glutCreateSubWindow(INT32 args)
{
  INT32 res;
  INT32 arg1;
  INT32 arg2;
  INT32 arg3;
  INT32 arg4;
  INT32 arg5;

  check_all_args("glutCreateSubWindow", args, BIT_INT, BIT_INT, BIT_INT, BIT_INT, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;
  arg3=Pike_sp[2-args].u.integer;
  arg4=Pike_sp[3-args].u.integer;
  arg5=Pike_sp[4-args].u.integer;

  res=glutCreateSubWindow(arg1,arg2,arg3,arg4,arg5);
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTCREATESUBWINDOW */

#ifndef MISSING_GLUTCREATEWINDOW
static void f_glutCreateWindow(INT32 args)
{
  INT32 res;
  char *arg1;

  check_all_args("glutCreateWindow", args, BIT_STRING, 0);

  arg1=Pike_sp[0-args].u.string->str;

  res=glutCreateWindow(arg1);
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTCREATEWINDOW */

#ifndef MISSING_GLUTDESTROYMENU
static void f_glutDestroyMenu(INT32 args)
{
  INT32 arg1;

  check_all_args("glutDestroyMenu", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutDestroyMenu(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTDESTROYMENU */

#ifndef MISSING_GLUTDESTROYWINDOW
static void f_glutDestroyWindow(INT32 args)
{
  INT32 arg1;

  check_all_args("glutDestroyWindow", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutDestroyWindow(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTDESTROYWINDOW */

#ifndef MISSING_GLUTDETACHMENU
static void f_glutDetachMenu(INT32 args)
{
  INT32 arg1;

  check_all_args("glutDetachMenu", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutDetachMenu(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTDETACHMENU */

#ifndef MISSING_GLUTDEVICEGET
static void f_glutDeviceGet(INT32 args)
{
  INT32 res;
  INT32 arg1;

  check_all_args("glutDeviceGet", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  res=glutDeviceGet(arg1);
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTDEVICEGET */

#ifndef MISSING_GLUTDIALSFUNC
struct svalue glutDialsFunc_fun;

void glutDialsFunc_cb_wrapper(int arg1, int arg2)
{
  push_int(arg1);
  push_int(arg2);
  apply_svalue(&glutDialsFunc_fun, 2);
  pop_stack();
}

static void f_glutDialsFunc(INT32 args)
{
  if(TYPEOF(glutDialsFunc_fun))
    free_svalue(&glutDialsFunc_fun);
  assign_svalue_no_free(&glutDialsFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutDialsFunc", args, BIT_FUNCTION, 0);
  glutDialsFunc((void *)&glutDialsFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTDIALSFUNC */

#ifndef MISSING_GLUTDISPLAYFUNC
struct svalue glutDisplayFunc_fun;

void glutDisplayFunc_cb_wrapper(void)
{

  apply_svalue(&glutDisplayFunc_fun, 0);
  pop_stack();
}

static void f_glutDisplayFunc(INT32 args)
{
  if(TYPEOF(glutDisplayFunc_fun))
    free_svalue(&glutDisplayFunc_fun);
  assign_svalue_no_free(&glutDisplayFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutDisplayFunc", args, BIT_FUNCTION, 0);
  glutDisplayFunc((void *)&glutDisplayFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTDISPLAYFUNC */

#ifndef MISSING_GLUTENTERGAMEMODE
static void f_glutEnterGameMode(INT32 args)
{
  INT32 res;
  res=glutEnterGameMode();
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTENTERGAMEMODE */

#ifndef MISSING_GLUTENTRYFUNC
struct svalue glutEntryFunc_fun;

void glutEntryFunc_cb_wrapper(int arg1)
{
  push_int(arg1);
  apply_svalue(&glutEntryFunc_fun, 1);
  pop_stack();
}

static void f_glutEntryFunc(INT32 args)
{
  if(TYPEOF(glutEntryFunc_fun))
    free_svalue(&glutEntryFunc_fun);
  assign_svalue_no_free(&glutEntryFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutEntryFunc", args, BIT_FUNCTION, 0);
  glutEntryFunc((void *)&glutEntryFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTENTRYFUNC */

#ifndef MISSING_GLUTESTABLISHOVERLAY
static void f_glutEstablishOverlay(INT32 args)
{
  glutEstablishOverlay();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTESTABLISHOVERLAY */

#ifndef MISSING_GLUTEXTENSIONSUPPORTED
static void f_glutExtensionSupported(INT32 args)
{
  INT32 res;
  char *arg1;

  check_all_args("glutExtensionSupported", args, BIT_STRING, 0);

  arg1=Pike_sp[0-args].u.string->str;

  res=glutExtensionSupported(arg1);
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTEXTENSIONSUPPORTED */

#ifndef MISSING_GLUTFORCEJOYSTICKFUNC
static void f_glutForceJoystickFunc(INT32 args)
{
  glutForceJoystickFunc();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTFORCEJOYSTICKFUNC */

#ifndef MISSING_GLUTFULLSCREEN
static void f_glutFullScreen(INT32 args)
{
  glutFullScreen();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTFULLSCREEN */

#ifndef MISSING_GLUTGAMEMODEGET
static void f_glutGameModeGet(INT32 args)
{
  INT32 res;
  INT32 arg1;

  check_all_args("glutGameModeGet", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  res=glutGameModeGet(arg1);
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTGAMEMODEGET */

#ifndef MISSING_GLUTGAMEMODESTRING
static void f_glutGameModeString(INT32 args)
{
  char *arg1;

  check_all_args("glutGameModeString", args, BIT_STRING, 0);

  arg1=Pike_sp[0-args].u.string->str;

  glutGameModeString(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTGAMEMODESTRING */

#ifndef MISSING_GLUTGET
static void f_glutGet(INT32 args)
{
  INT32 res;
  INT32 arg1;

  check_all_args("glutGet", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  res=glutGet(arg1);
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTGET */

#ifndef MISSING_GLUTGETCOLOR
static void f_glutGetColor(INT32 args)
{
  float res;
  INT32 arg1;
  INT32 arg2;

  check_all_args("glutGetColor", args, BIT_INT, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;

  res=glutGetColor(arg1,arg2);
  pop_n_elems(args);
  push_float(res);
}
#endif /* MISSING_GLUTGETCOLOR */

#ifndef MISSING_GLUTGETMENU
static void f_glutGetMenu(INT32 args)
{
  INT32 res;
  res=glutGetMenu();
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTGETMENU */

#ifndef MISSING_GLUTGETMODIFIERS
static void f_glutGetModifiers(INT32 args)
{
  INT32 res;
  res=glutGetModifiers();
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTGETMODIFIERS */

#ifndef MISSING_GLUTGETWINDOW
static void f_glutGetWindow(INT32 args)
{
  INT32 res;
  res=glutGetWindow();
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTGETWINDOW */

#ifndef MISSING_GLUTHIDEOVERLAY
static void f_glutHideOverlay(INT32 args)
{
  glutHideOverlay();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTHIDEOVERLAY */

#ifndef MISSING_GLUTHIDEWINDOW
static void f_glutHideWindow(INT32 args)
{
  glutHideWindow();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTHIDEWINDOW */

#ifndef MISSING_GLUTICONIFYWINDOW
static void f_glutIconifyWindow(INT32 args)
{
  glutIconifyWindow();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTICONIFYWINDOW */

#ifndef MISSING_GLUTIDLEFUNC
struct svalue glutIdleFunc_fun;

void glutIdleFunc_cb_wrapper(void)
{

  apply_svalue(&glutIdleFunc_fun, 0);
  pop_stack();
}

static void f_glutIdleFunc(INT32 args)
{
  if(TYPEOF(glutIdleFunc_fun))
    free_svalue(&glutIdleFunc_fun);
  assign_svalue_no_free(&glutIdleFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutIdleFunc", args, BIT_FUNCTION, 0);
  glutIdleFunc((void *)&glutIdleFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTIDLEFUNC */

#ifndef MISSING_GLUTIGNOREKEYREPEAT
static void f_glutIgnoreKeyRepeat(INT32 args)
{
  INT32 arg1;

  check_all_args("glutIgnoreKeyRepeat", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutIgnoreKeyRepeat(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTIGNOREKEYREPEAT */

#ifndef MISSING_GLUTINITDISPLAYMODE
static void f_glutInitDisplayMode(INT32 args)
{
  INT32 arg1;

  check_all_args("glutInitDisplayMode", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutInitDisplayMode(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTINITDISPLAYMODE */

#ifndef MISSING_GLUTINITDISPLAYSTRING
static void f_glutInitDisplayString(INT32 args)
{
  char *arg1;

  check_all_args("glutInitDisplayString", args, BIT_STRING, 0);

  arg1=Pike_sp[0-args].u.string->str;

  glutInitDisplayString(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTINITDISPLAYSTRING */

#ifndef MISSING_GLUTINITWINDOWPOSITION
static void f_glutInitWindowPosition(INT32 args)
{
  INT32 arg1;
  INT32 arg2;

  check_all_args("glutInitWindowPosition", args, BIT_INT, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;

  glutInitWindowPosition(arg1,arg2);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTINITWINDOWPOSITION */

#ifndef MISSING_GLUTINITWINDOWSIZE
static void f_glutInitWindowSize(INT32 args)
{
  INT32 arg1;
  INT32 arg2;

  check_all_args("glutInitWindowSize", args, BIT_INT, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;

  glutInitWindowSize(arg1,arg2);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTINITWINDOWSIZE */

#ifndef MISSING_GLUTJOYSTICKFUNC
struct svalue glutJoystickFunc_fun;

void glutJoystickFunc_cb_wrapper(unsigned int arg1, int arg2, int arg3, int arg4)
{
  push_int((int)arg1);
  push_int(arg2);
  push_int(arg3);
  push_int(arg4);
  apply_svalue(&glutJoystickFunc_fun, 4);
  pop_stack();
}

static void f_glutJoystickFunc(INT32 args)
{
  INT32 arg1;
  if(TYPEOF(glutJoystickFunc_fun))
    free_svalue(&glutJoystickFunc_fun);
  assign_svalue_no_free(&glutJoystickFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutJoystickFunc", args, BIT_FUNCTION, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutJoystickFunc((void *)&glutJoystickFunc_cb_wrapper,arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTJOYSTICKFUNC */

#ifndef MISSING_GLUTKEYBOARDFUNC
struct svalue glutKeyboardFunc_fun;

void glutKeyboardFunc_cb_wrapper(unsigned int arg1, int arg2, int arg3)
{
  push_int((int)arg1);
  push_int(arg2);
  push_int(arg3);
  apply_svalue(&glutKeyboardFunc_fun, 3);
  pop_stack();
}

static void f_glutKeyboardFunc(INT32 args)
{
  if(TYPEOF(glutKeyboardFunc_fun))
    free_svalue(&glutKeyboardFunc_fun);
  assign_svalue_no_free(&glutKeyboardFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutKeyboardFunc", args, BIT_FUNCTION, 0);
  glutKeyboardFunc((void *)&glutKeyboardFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTKEYBOARDFUNC */

#ifndef MISSING_GLUTKEYBOARDUPFUNC
struct svalue glutKeyboardUpFunc_fun;

void glutKeyboardUpFunc_cb_wrapper(unsigned int arg1, int arg2, int arg3)
{
  push_int((int)arg1);
  push_int(arg2);
  push_int(arg3);
  apply_svalue(&glutKeyboardUpFunc_fun, 3);
  pop_stack();
}

static void f_glutKeyboardUpFunc(INT32 args)
{
  if(TYPEOF(glutKeyboardUpFunc_fun))
    free_svalue(&glutKeyboardUpFunc_fun);
  assign_svalue_no_free(&glutKeyboardUpFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutKeyboardUpFunc", args, BIT_FUNCTION, 0);
  glutKeyboardUpFunc((void *)&glutKeyboardUpFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTKEYBOARDUPFUNC */

#ifndef MISSING_GLUTLAYERGET
static void f_glutLayerGet(INT32 args)
{
  INT32 res;
  INT32 arg1;

  check_all_args("glutLayerGet", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  res=glutLayerGet(arg1);
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTLAYERGET */

#ifndef MISSING_GLUTLEAVEGAMEMODE
static void f_glutLeaveGameMode(INT32 args)
{
  glutLeaveGameMode();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTLEAVEGAMEMODE */

#ifndef MISSING_GLUTMAINLOOP
static void f_glutMainLoop(INT32 args)
{
  glutMainLoop();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTMAINLOOP */

#ifndef MISSING_GLUTMENUSTATEFUNC
struct svalue glutMenuStateFunc_fun;

void glutMenuStateFunc_cb_wrapper(int arg1)
{
  push_int(arg1);
  apply_svalue(&glutMenuStateFunc_fun, 1);
  pop_stack();
}

static void f_glutMenuStateFunc(INT32 args)
{
  if(TYPEOF(glutMenuStateFunc_fun))
    free_svalue(&glutMenuStateFunc_fun);
  assign_svalue_no_free(&glutMenuStateFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutMenuStateFunc", args, BIT_FUNCTION, 0);
  glutMenuStateFunc((void *)&glutMenuStateFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTMENUSTATEFUNC */

#ifndef MISSING_GLUTMENUSTATUSFUNC
struct svalue glutMenuStatusFunc_fun;

void glutMenuStatusFunc_cb_wrapper(int arg1, int arg2, int arg3)
{
  push_int(arg1);
  push_int(arg2);
  push_int(arg3);
  apply_svalue(&glutMenuStatusFunc_fun, 3);
  pop_stack();
}

static void f_glutMenuStatusFunc(INT32 args)
{
  if(TYPEOF(glutMenuStatusFunc_fun))
    free_svalue(&glutMenuStatusFunc_fun);
  assign_svalue_no_free(&glutMenuStatusFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutMenuStatusFunc", args, BIT_FUNCTION, 0);
  glutMenuStatusFunc((void *)&glutMenuStatusFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTMENUSTATUSFUNC */

#ifndef MISSING_GLUTMOTIONFUNC
struct svalue glutMotionFunc_fun;

void glutMotionFunc_cb_wrapper(int arg1, int arg2)
{
  push_int(arg1);
  push_int(arg2);
  apply_svalue(&glutMotionFunc_fun, 2);
  pop_stack();
}

static void f_glutMotionFunc(INT32 args)
{
  if(TYPEOF(glutMotionFunc_fun))
    free_svalue(&glutMotionFunc_fun);
  assign_svalue_no_free(&glutMotionFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutMotionFunc", args, BIT_FUNCTION, 0);
  glutMotionFunc((void *)&glutMotionFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTMOTIONFUNC */

#ifndef MISSING_GLUTMOUSEFUNC
struct svalue glutMouseFunc_fun;

void glutMouseFunc_cb_wrapper(int arg1, int arg2, int arg3, int arg4)
{
  push_int(arg1);
  push_int(arg2);
  push_int(arg3);
  push_int(arg4);
  apply_svalue(&glutMouseFunc_fun, 4);
  pop_stack();
}

static void f_glutMouseFunc(INT32 args)
{
  if(TYPEOF(glutMouseFunc_fun))
    free_svalue(&glutMouseFunc_fun);
  assign_svalue_no_free(&glutMouseFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutMouseFunc", args, BIT_FUNCTION, 0);
  glutMouseFunc((void *)&glutMouseFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTMOUSEFUNC */

#ifndef MISSING_GLUTOVERLAYDISPLAYFUNC
struct svalue glutOverlayDisplayFunc_fun;

void glutOverlayDisplayFunc_cb_wrapper(void)
{

  apply_svalue(&glutOverlayDisplayFunc_fun, 0);
  pop_stack();
}

static void f_glutOverlayDisplayFunc(INT32 args)
{
  if(TYPEOF(glutOverlayDisplayFunc_fun))
    free_svalue(&glutOverlayDisplayFunc_fun);
  assign_svalue_no_free(&glutOverlayDisplayFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutOverlayDisplayFunc", args, BIT_FUNCTION, 0);
  glutOverlayDisplayFunc((void *)&glutOverlayDisplayFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTOVERLAYDISPLAYFUNC */

#ifndef MISSING_GLUTPASSIVEMOTIONFUNC
struct svalue glutPassiveMotionFunc_fun;

void glutPassiveMotionFunc_cb_wrapper(int arg1, int arg2)
{
  push_int(arg1);
  push_int(arg2);
  apply_svalue(&glutPassiveMotionFunc_fun, 2);
  pop_stack();
}

static void f_glutPassiveMotionFunc(INT32 args)
{
  if(TYPEOF(glutPassiveMotionFunc_fun))
    free_svalue(&glutPassiveMotionFunc_fun);
  assign_svalue_no_free(&glutPassiveMotionFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutPassiveMotionFunc", args, BIT_FUNCTION, 0);
  glutPassiveMotionFunc((void *)&glutPassiveMotionFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTPASSIVEMOTIONFUNC */

#ifndef MISSING_GLUTPOPWINDOW
static void f_glutPopWindow(INT32 args)
{
  glutPopWindow();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTPOPWINDOW */

#ifndef MISSING_GLUTPOSITIONWINDOW
static void f_glutPositionWindow(INT32 args)
{
  INT32 arg1;
  INT32 arg2;

  check_all_args("glutPositionWindow", args, BIT_INT, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;

  glutPositionWindow(arg1,arg2);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTPOSITIONWINDOW */

#ifndef MISSING_GLUTPOSTOVERLAYREDISPLAY
static void f_glutPostOverlayRedisplay(INT32 args)
{
  glutPostOverlayRedisplay();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTPOSTOVERLAYREDISPLAY */

#ifndef MISSING_GLUTPOSTREDISPLAY
static void f_glutPostRedisplay(INT32 args)
{
  glutPostRedisplay();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTPOSTREDISPLAY */

#ifndef MISSING_GLUTPOSTWINDOWOVERLAYREDISPLAY
static void f_glutPostWindowOverlayRedisplay(INT32 args)
{
  INT32 arg1;

  check_all_args("glutPostWindowOverlayRedisplay", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutPostWindowOverlayRedisplay(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTPOSTWINDOWOVERLAYREDISPLAY */

#ifndef MISSING_GLUTPOSTWINDOWREDISPLAY
static void f_glutPostWindowRedisplay(INT32 args)
{
  INT32 arg1;

  check_all_args("glutPostWindowRedisplay", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutPostWindowRedisplay(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTPOSTWINDOWREDISPLAY */

#ifndef MISSING_GLUTPUSHWINDOW
static void f_glutPushWindow(INT32 args)
{
  glutPushWindow();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTPUSHWINDOW */

#ifndef MISSING_GLUTREMOVEMENUITEM
static void f_glutRemoveMenuItem(INT32 args)
{
  INT32 arg1;

  check_all_args("glutRemoveMenuItem", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutRemoveMenuItem(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTREMOVEMENUITEM */

#ifndef MISSING_GLUTREMOVEOVERLAY
static void f_glutRemoveOverlay(INT32 args)
{
  glutRemoveOverlay();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTREMOVEOVERLAY */

#ifndef MISSING_GLUTREPORTERRORS
static void f_glutReportErrors(INT32 args)
{
  glutReportErrors();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTREPORTERRORS */

#ifndef MISSING_GLUTRESHAPEFUNC
struct svalue glutReshapeFunc_fun;

void glutReshapeFunc_cb_wrapper(int arg1, int arg2)
{
  push_int(arg1);
  push_int(arg2);
  apply_svalue(&glutReshapeFunc_fun, 2);
  pop_stack();
}

static void f_glutReshapeFunc(INT32 args)
{
  if(TYPEOF(glutReshapeFunc_fun))
    free_svalue(&glutReshapeFunc_fun);
  assign_svalue_no_free(&glutReshapeFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutReshapeFunc", args, BIT_FUNCTION, 0);
  glutReshapeFunc((void *)&glutReshapeFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTRESHAPEFUNC */

#ifndef MISSING_GLUTRESHAPEWINDOW
static void f_glutReshapeWindow(INT32 args)
{
  INT32 arg1;
  INT32 arg2;

  check_all_args("glutReshapeWindow", args, BIT_INT, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;

  glutReshapeWindow(arg1,arg2);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTRESHAPEWINDOW */

#ifndef MISSING_GLUTSETCOLOR
static void f_glutSetColor(INT32 args)
{
  INT32 arg1;
  float arg2;
  float arg3;
  float arg4;

  check_all_args("glutSetColor", args, BIT_INT, BIT_FLOAT, BIT_FLOAT, BIT_FLOAT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.float_number;
  arg3=Pike_sp[2-args].u.float_number;
  arg4=Pike_sp[3-args].u.float_number;

  glutSetColor(arg1,arg2,arg3,arg4);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSETCOLOR */

#ifndef MISSING_GLUTSETCURSOR
static void f_glutSetCursor(INT32 args)
{
  INT32 arg1;

  check_all_args("glutSetCursor", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutSetCursor(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSETCURSOR */

#ifndef MISSING_GLUTSETICONTITLE
static void f_glutSetIconTitle(INT32 args)
{
  char *arg1;

  check_all_args("glutSetIconTitle", args, BIT_STRING, 0);

  arg1=Pike_sp[0-args].u.string->str;

  glutSetIconTitle(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSETICONTITLE */

#ifndef MISSING_GLUTSETKEYREPEAT
static void f_glutSetKeyRepeat(INT32 args)
{
  INT32 arg1;

  check_all_args("glutSetKeyRepeat", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutSetKeyRepeat(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSETKEYREPEAT */

#ifndef MISSING_GLUTSETMENU
static void f_glutSetMenu(INT32 args)
{
  INT32 arg1;

  check_all_args("glutSetMenu", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutSetMenu(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSETMENU */

#ifndef MISSING_GLUTSETWINDOW
static void f_glutSetWindow(INT32 args)
{
  INT32 arg1;

  check_all_args("glutSetWindow", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutSetWindow(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSETWINDOW */

#ifndef MISSING_GLUTSETWINDOWTITLE
static void f_glutSetWindowTitle(INT32 args)
{
  char *arg1;

  check_all_args("glutSetWindowTitle", args, BIT_STRING, 0);

  arg1=Pike_sp[0-args].u.string->str;

  glutSetWindowTitle(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSETWINDOWTITLE */

#ifndef MISSING_GLUTSETUPVIDEORESIZING
static void f_glutSetupVideoResizing(INT32 args)
{
  glutSetupVideoResizing();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSETUPVIDEORESIZING */

#ifndef MISSING_GLUTSHOWOVERLAY
static void f_glutShowOverlay(INT32 args)
{
  glutShowOverlay();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSHOWOVERLAY */

#ifndef MISSING_GLUTSHOWWINDOW
static void f_glutShowWindow(INT32 args)
{
  glutShowWindow();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSHOWWINDOW */

#ifndef MISSING_GLUTSOLIDCONE
static void f_glutSolidCone(INT32 args)
{
  double arg1;
  double arg2;
  INT32 arg3;
  INT32 arg4;

  check_all_args("glutSolidCone", args, BIT_FLOAT, BIT_FLOAT, BIT_INT, BIT_INT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;
  arg2=(double)Pike_sp[1-args].u.float_number;
  arg3=Pike_sp[2-args].u.integer;
  arg4=Pike_sp[3-args].u.integer;

  glutSolidCone(arg1,arg2,arg3,arg4);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSOLIDCONE */

#ifndef MISSING_GLUTSOLIDCUBE
static void f_glutSolidCube(INT32 args)
{
  double arg1;

  check_all_args("glutSolidCube", args, BIT_FLOAT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;

  glutSolidCube(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSOLIDCUBE */

#ifndef MISSING_GLUTSOLIDDODECAHEDRON
static void f_glutSolidDodecahedron(INT32 args)
{
  glutSolidDodecahedron();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSOLIDDODECAHEDRON */

#ifndef MISSING_GLUTSOLIDICOSAHEDRON
static void f_glutSolidIcosahedron(INT32 args)
{
  glutSolidIcosahedron();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSOLIDICOSAHEDRON */

#ifndef MISSING_GLUTSOLIDOCTAHEDRON
static void f_glutSolidOctahedron(INT32 args)
{
  glutSolidOctahedron();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSOLIDOCTAHEDRON */

#ifndef MISSING_GLUTSOLIDSPHERE
static void f_glutSolidSphere(INT32 args)
{
  double arg1;
  INT32 arg2;
  INT32 arg3;

  check_all_args("glutSolidSphere", args, BIT_FLOAT, BIT_INT, BIT_INT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;
  arg2=Pike_sp[1-args].u.integer;
  arg3=Pike_sp[2-args].u.integer;

  glutSolidSphere(arg1,arg2,arg3);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSOLIDSPHERE */

#ifndef MISSING_GLUTSOLIDTEAPOT
static void f_glutSolidTeapot(INT32 args)
{
  double arg1;

  check_all_args("glutSolidTeapot", args, BIT_FLOAT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;

  glutSolidTeapot(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSOLIDTEAPOT */

#ifndef MISSING_GLUTSOLIDTETRAHEDRON
static void f_glutSolidTetrahedron(INT32 args)
{
  glutSolidTetrahedron();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSOLIDTETRAHEDRON */

#ifndef MISSING_GLUTSOLIDTORUS
static void f_glutSolidTorus(INT32 args)
{
  double arg1;
  double arg2;
  INT32 arg3;
  INT32 arg4;

  check_all_args("glutSolidTorus", args, BIT_FLOAT, BIT_FLOAT, BIT_INT, BIT_INT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;
  arg2=(double)Pike_sp[1-args].u.float_number;
  arg3=Pike_sp[2-args].u.integer;
  arg4=Pike_sp[3-args].u.integer;

  glutSolidTorus(arg1,arg2,arg3,arg4);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSOLIDTORUS */

#ifndef MISSING_GLUTSPACEBALLBUTTONFUNC
struct svalue glutSpaceballButtonFunc_fun;

void glutSpaceballButtonFunc_cb_wrapper(int arg1, int arg2)
{
  push_int(arg1);
  push_int(arg2);
  apply_svalue(&glutSpaceballButtonFunc_fun, 2);
  pop_stack();
}

static void f_glutSpaceballButtonFunc(INT32 args)
{
  if(TYPEOF(glutSpaceballButtonFunc_fun))
    free_svalue(&glutSpaceballButtonFunc_fun);
  assign_svalue_no_free(&glutSpaceballButtonFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutSpaceballButtonFunc", args, BIT_FUNCTION, 0);
  glutSpaceballButtonFunc((void *)&glutSpaceballButtonFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSPACEBALLBUTTONFUNC */

#ifndef MISSING_GLUTSPACEBALLMOTIONFUNC
struct svalue glutSpaceballMotionFunc_fun;

void glutSpaceballMotionFunc_cb_wrapper(int arg1, int arg2, int arg3)
{
  push_int(arg1);
  push_int(arg2);
  push_int(arg3);
  apply_svalue(&glutSpaceballMotionFunc_fun, 3);
  pop_stack();
}

static void f_glutSpaceballMotionFunc(INT32 args)
{
  if(TYPEOF(glutSpaceballMotionFunc_fun))
    free_svalue(&glutSpaceballMotionFunc_fun);
  assign_svalue_no_free(&glutSpaceballMotionFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutSpaceballMotionFunc", args, BIT_FUNCTION, 0);
  glutSpaceballMotionFunc((void *)&glutSpaceballMotionFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSPACEBALLMOTIONFUNC */

#ifndef MISSING_GLUTSPACEBALLROTATEFUNC
struct svalue glutSpaceballRotateFunc_fun;

void glutSpaceballRotateFunc_cb_wrapper(int arg1, int arg2, int arg3)
{
  push_int(arg1);
  push_int(arg2);
  push_int(arg3);
  apply_svalue(&glutSpaceballRotateFunc_fun, 3);
  pop_stack();
}

static void f_glutSpaceballRotateFunc(INT32 args)
{
  if(TYPEOF(glutSpaceballRotateFunc_fun))
    free_svalue(&glutSpaceballRotateFunc_fun);
  assign_svalue_no_free(&glutSpaceballRotateFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutSpaceballRotateFunc", args, BIT_FUNCTION, 0);
  glutSpaceballRotateFunc((void *)&glutSpaceballRotateFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSPACEBALLROTATEFUNC */

#ifndef MISSING_GLUTSPECIALFUNC
struct svalue glutSpecialFunc_fun;

void glutSpecialFunc_cb_wrapper(int arg1, int arg2, int arg3)
{
  push_int(arg1);
  push_int(arg2);
  push_int(arg3);
  apply_svalue(&glutSpecialFunc_fun, 3);
  pop_stack();
}

static void f_glutSpecialFunc(INT32 args)
{
  if(TYPEOF(glutSpecialFunc_fun))
    free_svalue(&glutSpecialFunc_fun);
  assign_svalue_no_free(&glutSpecialFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutSpecialFunc", args, BIT_FUNCTION, 0);
  glutSpecialFunc((void *)&glutSpecialFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSPECIALFUNC */

#ifndef MISSING_GLUTSPECIALUPFUNC
struct svalue glutSpecialUpFunc_fun;

void glutSpecialUpFunc_cb_wrapper(int arg1, int arg2, int arg3)
{
  push_int(arg1);
  push_int(arg2);
  push_int(arg3);
  apply_svalue(&glutSpecialUpFunc_fun, 3);
  pop_stack();
}

static void f_glutSpecialUpFunc(INT32 args)
{
  if(TYPEOF(glutSpecialUpFunc_fun))
    free_svalue(&glutSpecialUpFunc_fun);
  assign_svalue_no_free(&glutSpecialUpFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutSpecialUpFunc", args, BIT_FUNCTION, 0);
  glutSpecialUpFunc((void *)&glutSpecialUpFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSPECIALUPFUNC */

#ifndef MISSING_GLUTSTOPVIDEORESIZING
static void f_glutStopVideoResizing(INT32 args)
{
  glutStopVideoResizing();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSTOPVIDEORESIZING */

#ifndef MISSING_GLUTSWAPBUFFERS
static void f_glutSwapBuffers(INT32 args)
{
  glutSwapBuffers();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTSWAPBUFFERS */

#ifndef MISSING_GLUTTABLETBUTTONFUNC
struct svalue glutTabletButtonFunc_fun;

void glutTabletButtonFunc_cb_wrapper(int arg1, int arg2, int arg3, int arg4)
{
  push_int(arg1);
  push_int(arg2);
  push_int(arg3);
  push_int(arg4);
  apply_svalue(&glutTabletButtonFunc_fun, 4);
  pop_stack();
}

static void f_glutTabletButtonFunc(INT32 args)
{
  if(TYPEOF(glutTabletButtonFunc_fun))
    free_svalue(&glutTabletButtonFunc_fun);
  assign_svalue_no_free(&glutTabletButtonFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutTabletButtonFunc", args, BIT_FUNCTION, 0);
  glutTabletButtonFunc((void *)&glutTabletButtonFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTTABLETBUTTONFUNC */

#ifndef MISSING_GLUTTABLETMOTIONFUNC
struct svalue glutTabletMotionFunc_fun;

void glutTabletMotionFunc_cb_wrapper(int arg1, int arg2)
{
  push_int(arg1);
  push_int(arg2);
  apply_svalue(&glutTabletMotionFunc_fun, 2);
  pop_stack();
}

static void f_glutTabletMotionFunc(INT32 args)
{
  if(TYPEOF(glutTabletMotionFunc_fun))
    free_svalue(&glutTabletMotionFunc_fun);
  assign_svalue_no_free(&glutTabletMotionFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutTabletMotionFunc", args, BIT_FUNCTION, 0);
  glutTabletMotionFunc((void *)&glutTabletMotionFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTTABLETMOTIONFUNC */

#ifndef MISSING_GLUTTIMERFUNC
struct svalue glutTimerFunc_fun;

void glutTimerFunc_cb_wrapper(int arg1)
{
  push_int(arg1);
  apply_svalue(&glutTimerFunc_fun, 1);
  pop_stack();
}

static void f_glutTimerFunc(INT32 args)
{
  INT32 arg1;
  INT32 arg2;
  if(TYPEOF(glutTimerFunc_fun))
    free_svalue(&glutTimerFunc_fun);
  assign_svalue_no_free(&glutTimerFunc_fun, &Pike_sp[-args+1]);

  check_all_args("glutTimerFunc", args, BIT_INT, BIT_FUNCTION, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;

  glutTimerFunc(arg1,(void *)&glutTimerFunc_cb_wrapper,arg2);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTTIMERFUNC */

#ifndef MISSING_GLUTUSELAYER
static void f_glutUseLayer(INT32 args)
{
  INT32 arg1;

  check_all_args("glutUseLayer", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  glutUseLayer(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTUSELAYER */

#ifndef MISSING_GLUTVIDEOPAN
static void f_glutVideoPan(INT32 args)
{
  INT32 arg1;
  INT32 arg2;
  INT32 arg3;
  INT32 arg4;

  check_all_args("glutVideoPan", args, BIT_INT, BIT_INT, BIT_INT, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;
  arg3=Pike_sp[2-args].u.integer;
  arg4=Pike_sp[3-args].u.integer;

  glutVideoPan(arg1,arg2,arg3,arg4);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTVIDEOPAN */

#ifndef MISSING_GLUTVIDEORESIZE
static void f_glutVideoResize(INT32 args)
{
  INT32 arg1;
  INT32 arg2;
  INT32 arg3;
  INT32 arg4;

  check_all_args("glutVideoResize", args, BIT_INT, BIT_INT, BIT_INT, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;
  arg3=Pike_sp[2-args].u.integer;
  arg4=Pike_sp[3-args].u.integer;

  glutVideoResize(arg1,arg2,arg3,arg4);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTVIDEORESIZE */

#ifndef MISSING_GLUTVIDEORESIZEGET
static void f_glutVideoResizeGet(INT32 args)
{
  INT32 res;
  INT32 arg1;

  check_all_args("glutVideoResizeGet", args, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;

  res=glutVideoResizeGet(arg1);
  pop_n_elems(args);
  push_int(res);
}
#endif /* MISSING_GLUTVIDEORESIZEGET */

#ifndef MISSING_GLUTVISIBILITYFUNC
struct svalue glutVisibilityFunc_fun;

void glutVisibilityFunc_cb_wrapper(int arg1)
{
  push_int(arg1);
  apply_svalue(&glutVisibilityFunc_fun, 1);
  pop_stack();
}

static void f_glutVisibilityFunc(INT32 args)
{
  if(TYPEOF(glutVisibilityFunc_fun))
    free_svalue(&glutVisibilityFunc_fun);
  assign_svalue_no_free(&glutVisibilityFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutVisibilityFunc", args, BIT_FUNCTION, 0);
  glutVisibilityFunc((void *)&glutVisibilityFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTVISIBILITYFUNC */

#ifndef MISSING_GLUTWARPPOINTER
static void f_glutWarpPointer(INT32 args)
{
  INT32 arg1;
  INT32 arg2;

  check_all_args("glutWarpPointer", args, BIT_INT, BIT_INT, 0);

  arg1=Pike_sp[0-args].u.integer;
  arg2=Pike_sp[1-args].u.integer;

  glutWarpPointer(arg1,arg2);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWARPPOINTER */

#ifndef MISSING_GLUTWINDOWSTATUSFUNC
struct svalue glutWindowStatusFunc_fun;

void glutWindowStatusFunc_cb_wrapper(int arg1)
{
  push_int(arg1);
  apply_svalue(&glutWindowStatusFunc_fun, 1);
  pop_stack();
}

static void f_glutWindowStatusFunc(INT32 args)
{
  if(TYPEOF(glutWindowStatusFunc_fun))
    free_svalue(&glutWindowStatusFunc_fun);
  assign_svalue_no_free(&glutWindowStatusFunc_fun, &Pike_sp[-args+0]);

  check_all_args("glutWindowStatusFunc", args, BIT_FUNCTION, 0);
  glutWindowStatusFunc((void *)&glutWindowStatusFunc_cb_wrapper);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWINDOWSTATUSFUNC */

#ifndef MISSING_GLUTWIRECONE
static void f_glutWireCone(INT32 args)
{
  double arg1;
  double arg2;
  INT32 arg3;
  INT32 arg4;

  check_all_args("glutWireCone", args, BIT_FLOAT, BIT_FLOAT, BIT_INT, BIT_INT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;
  arg2=(double)Pike_sp[1-args].u.float_number;
  arg3=Pike_sp[2-args].u.integer;
  arg4=Pike_sp[3-args].u.integer;

  glutWireCone(arg1,arg2,arg3,arg4);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWIRECONE */

#ifndef MISSING_GLUTWIRECUBE
static void f_glutWireCube(INT32 args)
{
  double arg1;

  check_all_args("glutWireCube", args, BIT_FLOAT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;

  glutWireCube(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWIRECUBE */

#ifndef MISSING_GLUTWIREDODECAHEDRON
static void f_glutWireDodecahedron(INT32 args)
{
  glutWireDodecahedron();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWIREDODECAHEDRON */

#ifndef MISSING_GLUTWIREICOSAHEDRON
static void f_glutWireIcosahedron(INT32 args)
{
  glutWireIcosahedron();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWIREICOSAHEDRON */

#ifndef MISSING_GLUTWIREOCTAHEDRON
static void f_glutWireOctahedron(INT32 args)
{
  glutWireOctahedron();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWIREOCTAHEDRON */

#ifndef MISSING_GLUTWIRESPHERE
static void f_glutWireSphere(INT32 args)
{
  double arg1;
  INT32 arg2;
  INT32 arg3;

  check_all_args("glutWireSphere", args, BIT_FLOAT, BIT_INT, BIT_INT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;
  arg2=Pike_sp[1-args].u.integer;
  arg3=Pike_sp[2-args].u.integer;

  glutWireSphere(arg1,arg2,arg3);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWIRESPHERE */

#ifndef MISSING_GLUTWIRETEAPOT
static void f_glutWireTeapot(INT32 args)
{
  double arg1;

  check_all_args("glutWireTeapot", args, BIT_FLOAT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;

  glutWireTeapot(arg1);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWIRETEAPOT */

#ifndef MISSING_GLUTWIRETETRAHEDRON
static void f_glutWireTetrahedron(INT32 args)
{
  glutWireTetrahedron();
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWIRETETRAHEDRON */

#ifndef MISSING_GLUTWIRETORUS
static void f_glutWireTorus(INT32 args)
{
  double arg1;
  double arg2;
  INT32 arg3;
  INT32 arg4;

  check_all_args("glutWireTorus", args, BIT_FLOAT, BIT_FLOAT, BIT_INT, BIT_INT, 0);

  arg1=(double)Pike_sp[0-args].u.float_number;
  arg2=(double)Pike_sp[1-args].u.float_number;
  arg3=Pike_sp[2-args].u.integer;
  arg4=Pike_sp[3-args].u.integer;

  glutWireTorus(arg1,arg2,arg3,arg4);
  pop_n_elems(args);
}
#endif /* MISSING_GLUTWIRETORUS */

void add_auto_funcs_glut()
{
  pre_init();
#ifndef MISSING_GLUTADDMENUENTRY
  add_function_constant("glutAddMenuEntry", f_glutAddMenuEntry,
			"function(string,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTADDSUBMENU
  add_function_constant("glutAddSubMenu", f_glutAddSubMenu,
			"function(string,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTATTACHMENU
  add_function_constant("glutAttachMenu", f_glutAttachMenu,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTBUTTONBOXFUNC
  add_function_constant("glutButtonBoxFunc", f_glutButtonBoxFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTCHANGETOMENUENTRY
  add_function_constant("glutChangeToMenuEntry", f_glutChangeToMenuEntry,
			"function(int,string,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTCHANGETOSUBMENU
  add_function_constant("glutChangeToSubMenu", f_glutChangeToSubMenu,
			"function(int,string,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTCOPYCOLORMAP
  add_function_constant("glutCopyColormap", f_glutCopyColormap,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTCREATEMENU
  add_function_constant("glutCreateMenu", f_glutCreateMenu,
			"function(function:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTCREATESUBWINDOW
  add_function_constant("glutCreateSubWindow", f_glutCreateSubWindow,
			"function(int,int,int,int,int:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTCREATEWINDOW
  add_function_constant("glutCreateWindow", f_glutCreateWindow,
			"function(string:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTDESTROYMENU
  add_function_constant("glutDestroyMenu", f_glutDestroyMenu,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTDESTROYWINDOW
  add_function_constant("glutDestroyWindow", f_glutDestroyWindow,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTDETACHMENU
  add_function_constant("glutDetachMenu", f_glutDetachMenu,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTDEVICEGET
  add_function_constant("glutDeviceGet", f_glutDeviceGet,
			"function(int:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTDIALSFUNC
  add_function_constant("glutDialsFunc", f_glutDialsFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTDISPLAYFUNC
  add_function_constant("glutDisplayFunc", f_glutDisplayFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTENTERGAMEMODE
  add_function_constant("glutEnterGameMode", f_glutEnterGameMode,
			"function(:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTENTRYFUNC
  add_function_constant("glutEntryFunc", f_glutEntryFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTESTABLISHOVERLAY
  add_function_constant("glutEstablishOverlay", f_glutEstablishOverlay,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTEXTENSIONSUPPORTED
  add_function_constant("glutExtensionSupported", f_glutExtensionSupported,
			"function(string:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTFORCEJOYSTICKFUNC
  add_function_constant("glutForceJoystickFunc", f_glutForceJoystickFunc,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTFULLSCREEN
  add_function_constant("glutFullScreen", f_glutFullScreen,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTGAMEMODEGET
  add_function_constant("glutGameModeGet", f_glutGameModeGet,
			"function(int:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTGAMEMODESTRING
  add_function_constant("glutGameModeString", f_glutGameModeString,
			"function(string:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTGET
  add_function_constant("glutGet", f_glutGet,
			"function(int:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTGETCOLOR
  add_function_constant("glutGetColor", f_glutGetColor,
			"function(int,int:float)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTGETMENU
  add_function_constant("glutGetMenu", f_glutGetMenu,
			"function(:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTGETMODIFIERS
  add_function_constant("glutGetModifiers", f_glutGetModifiers,
			"function(:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTGETWINDOW
  add_function_constant("glutGetWindow", f_glutGetWindow,
			"function(:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTHIDEOVERLAY
  add_function_constant("glutHideOverlay", f_glutHideOverlay,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTHIDEWINDOW
  add_function_constant("glutHideWindow", f_glutHideWindow,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTICONIFYWINDOW
  add_function_constant("glutIconifyWindow", f_glutIconifyWindow,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTIDLEFUNC
  add_function_constant("glutIdleFunc", f_glutIdleFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTIGNOREKEYREPEAT
  add_function_constant("glutIgnoreKeyRepeat", f_glutIgnoreKeyRepeat,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTINITDISPLAYMODE
  add_function_constant("glutInitDisplayMode", f_glutInitDisplayMode,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTINITDISPLAYSTRING
  add_function_constant("glutInitDisplayString", f_glutInitDisplayString,
			"function(string:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTINITWINDOWPOSITION
  add_function_constant("glutInitWindowPosition", f_glutInitWindowPosition,
			"function(int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTINITWINDOWSIZE
  add_function_constant("glutInitWindowSize", f_glutInitWindowSize,
			"function(int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTJOYSTICKFUNC
  add_function_constant("glutJoystickFunc", f_glutJoystickFunc,
			"function(function,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTKEYBOARDFUNC
  add_function_constant("glutKeyboardFunc", f_glutKeyboardFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTKEYBOARDUPFUNC
  add_function_constant("glutKeyboardUpFunc", f_glutKeyboardUpFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTLAYERGET
  add_function_constant("glutLayerGet", f_glutLayerGet,
			"function(int:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTLEAVEGAMEMODE
  add_function_constant("glutLeaveGameMode", f_glutLeaveGameMode,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTMAINLOOP
  add_function_constant("glutMainLoop", f_glutMainLoop,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTMENUSTATEFUNC
  add_function_constant("glutMenuStateFunc", f_glutMenuStateFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTMENUSTATUSFUNC
  add_function_constant("glutMenuStatusFunc", f_glutMenuStatusFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTMOTIONFUNC
  add_function_constant("glutMotionFunc", f_glutMotionFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTMOUSEFUNC
  add_function_constant("glutMouseFunc", f_glutMouseFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTOVERLAYDISPLAYFUNC
  add_function_constant("glutOverlayDisplayFunc", f_glutOverlayDisplayFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTPASSIVEMOTIONFUNC
  add_function_constant("glutPassiveMotionFunc", f_glutPassiveMotionFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTPOPWINDOW
  add_function_constant("glutPopWindow", f_glutPopWindow,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTPOSITIONWINDOW
  add_function_constant("glutPositionWindow", f_glutPositionWindow,
			"function(int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTPOSTOVERLAYREDISPLAY
  add_function_constant("glutPostOverlayRedisplay", f_glutPostOverlayRedisplay,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTPOSTREDISPLAY
  add_function_constant("glutPostRedisplay", f_glutPostRedisplay,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTPOSTWINDOWOVERLAYREDISPLAY
  add_function_constant("glutPostWindowOverlayRedisplay", f_glutPostWindowOverlayRedisplay,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTPOSTWINDOWREDISPLAY
  add_function_constant("glutPostWindowRedisplay", f_glutPostWindowRedisplay,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTPUSHWINDOW
  add_function_constant("glutPushWindow", f_glutPushWindow,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTREMOVEMENUITEM
  add_function_constant("glutRemoveMenuItem", f_glutRemoveMenuItem,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTREMOVEOVERLAY
  add_function_constant("glutRemoveOverlay", f_glutRemoveOverlay,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTREPORTERRORS
  add_function_constant("glutReportErrors", f_glutReportErrors,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTRESHAPEFUNC
  add_function_constant("glutReshapeFunc", f_glutReshapeFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTRESHAPEWINDOW
  add_function_constant("glutReshapeWindow", f_glutReshapeWindow,
			"function(int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSETCOLOR
  add_function_constant("glutSetColor", f_glutSetColor,
			"function(int,float,float,float:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSETCURSOR
  add_function_constant("glutSetCursor", f_glutSetCursor,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSETICONTITLE
  add_function_constant("glutSetIconTitle", f_glutSetIconTitle,
			"function(string:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSETKEYREPEAT
  add_function_constant("glutSetKeyRepeat", f_glutSetKeyRepeat,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSETMENU
  add_function_constant("glutSetMenu", f_glutSetMenu,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSETWINDOW
  add_function_constant("glutSetWindow", f_glutSetWindow,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSETWINDOWTITLE
  add_function_constant("glutSetWindowTitle", f_glutSetWindowTitle,
			"function(string:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSETUPVIDEORESIZING
  add_function_constant("glutSetupVideoResizing", f_glutSetupVideoResizing,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSHOWOVERLAY
  add_function_constant("glutShowOverlay", f_glutShowOverlay,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSHOWWINDOW
  add_function_constant("glutShowWindow", f_glutShowWindow,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSOLIDCONE
  add_function_constant("glutSolidCone", f_glutSolidCone,
			"function(float,float,int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSOLIDCUBE
  add_function_constant("glutSolidCube", f_glutSolidCube,
			"function(float:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSOLIDDODECAHEDRON
  add_function_constant("glutSolidDodecahedron", f_glutSolidDodecahedron,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSOLIDICOSAHEDRON
  add_function_constant("glutSolidIcosahedron", f_glutSolidIcosahedron,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSOLIDOCTAHEDRON
  add_function_constant("glutSolidOctahedron", f_glutSolidOctahedron,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSOLIDSPHERE
  add_function_constant("glutSolidSphere", f_glutSolidSphere,
			"function(float,int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSOLIDTEAPOT
  add_function_constant("glutSolidTeapot", f_glutSolidTeapot,
			"function(float:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSOLIDTETRAHEDRON
  add_function_constant("glutSolidTetrahedron", f_glutSolidTetrahedron,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSOLIDTORUS
  add_function_constant("glutSolidTorus", f_glutSolidTorus,
			"function(float,float,int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSPACEBALLBUTTONFUNC
  add_function_constant("glutSpaceballButtonFunc", f_glutSpaceballButtonFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSPACEBALLMOTIONFUNC
  add_function_constant("glutSpaceballMotionFunc", f_glutSpaceballMotionFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSPACEBALLROTATEFUNC
  add_function_constant("glutSpaceballRotateFunc", f_glutSpaceballRotateFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSPECIALFUNC
  add_function_constant("glutSpecialFunc", f_glutSpecialFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSPECIALUPFUNC
  add_function_constant("glutSpecialUpFunc", f_glutSpecialUpFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSTOPVIDEORESIZING
  add_function_constant("glutStopVideoResizing", f_glutStopVideoResizing,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTSWAPBUFFERS
  add_function_constant("glutSwapBuffers", f_glutSwapBuffers,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTTABLETBUTTONFUNC
  add_function_constant("glutTabletButtonFunc", f_glutTabletButtonFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTTABLETMOTIONFUNC
  add_function_constant("glutTabletMotionFunc", f_glutTabletMotionFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTTIMERFUNC
  add_function_constant("glutTimerFunc", f_glutTimerFunc,
			"function(int,function,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTUSELAYER
  add_function_constant("glutUseLayer", f_glutUseLayer,
			"function(int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTVIDEOPAN
  add_function_constant("glutVideoPan", f_glutVideoPan,
			"function(int,int,int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTVIDEORESIZE
  add_function_constant("glutVideoResize", f_glutVideoResize,
			"function(int,int,int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTVIDEORESIZEGET
  add_function_constant("glutVideoResizeGet", f_glutVideoResizeGet,
			"function(int:int)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTVISIBILITYFUNC
  add_function_constant("glutVisibilityFunc", f_glutVisibilityFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWARPPOINTER
  add_function_constant("glutWarpPointer", f_glutWarpPointer,
			"function(int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWINDOWSTATUSFUNC
  add_function_constant("glutWindowStatusFunc", f_glutWindowStatusFunc,
			"function(function:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWIRECONE
  add_function_constant("glutWireCone", f_glutWireCone,
			"function(float,float,int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWIRECUBE
  add_function_constant("glutWireCube", f_glutWireCube,
			"function(float:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWIREDODECAHEDRON
  add_function_constant("glutWireDodecahedron", f_glutWireDodecahedron,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWIREICOSAHEDRON
  add_function_constant("glutWireIcosahedron", f_glutWireIcosahedron,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWIREOCTAHEDRON
  add_function_constant("glutWireOctahedron", f_glutWireOctahedron,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWIRESPHERE
  add_function_constant("glutWireSphere", f_glutWireSphere,
			"function(float,int,int:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWIRETEAPOT
  add_function_constant("glutWireTeapot", f_glutWireTeapot,
			"function(float:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWIRETETRAHEDRON
  add_function_constant("glutWireTetrahedron", f_glutWireTetrahedron,
			"function(:void)", OPT_SIDE_EFFECT);
#endif
#ifndef MISSING_GLUTWIRETORUS
  add_function_constant("glutWireTorus", f_glutWireTorus,
			"function(float,float,int,int:void)", OPT_SIDE_EFFECT);
#endif
  add_integer_constant("GLUT_ACCUM", 4, ID_FINAL);
  add_integer_constant("GLUT_ACTIVE_ALT", 4, ID_FINAL);
  add_integer_constant("GLUT_ACTIVE_CTRL", 2, ID_FINAL);
  add_integer_constant("GLUT_ACTIVE_SHIFT", 1, ID_FINAL);
  add_integer_constant("GLUT_ALPHA", 8, ID_FINAL);
  add_integer_constant("GLUT_BLUE", 2, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_BOTTOM_LEFT_CORNER", 19, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_BOTTOM_RIGHT_CORNER", 18, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_BOTTOM_SIDE", 13, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_CROSSHAIR", 9, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_CYCLE", 5, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_DESTROY", 3, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_FULL_CROSSHAIR", 102, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_HELP", 4, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_INFO", 2, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_INHERIT", 100, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_LEFT_ARROW", 1, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_LEFT_RIGHT", 11, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_LEFT_SIDE", 14, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_NONE", 101, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_RIGHT_ARROW", 0, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_RIGHT_SIDE", 15, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_SPRAY", 6, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_TEXT", 8, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_TOP_LEFT_CORNER", 16, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_TOP_RIGHT_CORNER", 17, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_TOP_SIDE", 12, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_UP_DOWN", 10, ID_FINAL);
  add_integer_constant("GLUT_CURSOR_WAIT", 7, ID_FINAL);
  add_integer_constant("GLUT_DEPTH", 16, ID_FINAL);
  add_integer_constant("GLUT_DEVICE_IGNORE_KEY_REPEAT", 610, ID_FINAL);
  add_integer_constant("GLUT_DEVICE_KEY_REPEAT", 611, ID_FINAL);
  add_integer_constant("GLUT_DISPLAY_MODE_POSSIBLE", 400, ID_FINAL);
  add_integer_constant("GLUT_DOUBLE", 2, ID_FINAL);
  add_integer_constant("GLUT_DOWN", 0, ID_FINAL);
  add_integer_constant("GLUT_ELAPSED_TIME", 700, ID_FINAL);
  add_integer_constant("GLUT_ENTERED", 1, ID_FINAL);
  add_integer_constant("GLUT_FULLY_COVERED", 3, ID_FINAL);
  add_integer_constant("GLUT_FULLY_RETAINED", 1, ID_FINAL);
  add_integer_constant("GLUT_GAME_MODE_ACTIVE", 0, ID_FINAL);
  add_integer_constant("GLUT_GAME_MODE_DISPLAY_CHANGED", 6, ID_FINAL);
  add_integer_constant("GLUT_GAME_MODE_HEIGHT", 3, ID_FINAL);
  add_integer_constant("GLUT_GAME_MODE_PIXEL_DEPTH", 4, ID_FINAL);
  add_integer_constant("GLUT_GAME_MODE_POSSIBLE", 1, ID_FINAL);
  add_integer_constant("GLUT_GAME_MODE_REFRESH_RATE", 5, ID_FINAL);
  add_integer_constant("GLUT_GAME_MODE_WIDTH", 2, ID_FINAL);
  add_integer_constant("GLUT_GREEN", 1, ID_FINAL);
  add_integer_constant("GLUT_HAS_DIAL_AND_BUTTON_BOX", 603, ID_FINAL);
  add_integer_constant("GLUT_HAS_JOYSTICK", 612, ID_FINAL);
  add_integer_constant("GLUT_HAS_KEYBOARD", 600, ID_FINAL);
  add_integer_constant("GLUT_HAS_MOUSE", 601, ID_FINAL);
  add_integer_constant("GLUT_HAS_OVERLAY", 802, ID_FINAL);
  add_integer_constant("GLUT_HAS_SPACEBALL", 602, ID_FINAL);
  add_integer_constant("GLUT_HAS_TABLET", 604, ID_FINAL);
  add_integer_constant("GLUT_HIDDEN", 0, ID_FINAL);
  add_integer_constant("GLUT_INDEX", 1, ID_FINAL);
  add_integer_constant("GLUT_INIT_DISPLAY_MODE", 504, ID_FINAL);
  add_integer_constant("GLUT_INIT_WINDOW_HEIGHT", 503, ID_FINAL);
  add_integer_constant("GLUT_INIT_WINDOW_WIDTH", 502, ID_FINAL);
  add_integer_constant("GLUT_INIT_WINDOW_X", 500, ID_FINAL);
  add_integer_constant("GLUT_INIT_WINDOW_Y", 501, ID_FINAL);
  add_integer_constant("GLUT_JOYSTICK_AXES", 615, ID_FINAL);
  add_integer_constant("GLUT_JOYSTICK_BUTTONS", 614, ID_FINAL);
  add_integer_constant("GLUT_JOYSTICK_BUTTON_A", 1, ID_FINAL);
  add_integer_constant("GLUT_JOYSTICK_BUTTON_B", 2, ID_FINAL);
  add_integer_constant("GLUT_JOYSTICK_BUTTON_C", 4, ID_FINAL);
  add_integer_constant("GLUT_JOYSTICK_BUTTON_D", 8, ID_FINAL);
  add_integer_constant("GLUT_JOYSTICK_POLL_RATE", 616, ID_FINAL);
  add_integer_constant("GLUT_KEY_DOWN", 103, ID_FINAL);
  add_integer_constant("GLUT_KEY_END", 107, ID_FINAL);
  add_integer_constant("GLUT_KEY_F1", 1, ID_FINAL);
  add_integer_constant("GLUT_KEY_F10", 10, ID_FINAL);
  add_integer_constant("GLUT_KEY_F11", 11, ID_FINAL);
  add_integer_constant("GLUT_KEY_F12", 12, ID_FINAL);
  add_integer_constant("GLUT_KEY_F2", 2, ID_FINAL);
  add_integer_constant("GLUT_KEY_F3", 3, ID_FINAL);
  add_integer_constant("GLUT_KEY_F4", 4, ID_FINAL);
  add_integer_constant("GLUT_KEY_F5", 5, ID_FINAL);
  add_integer_constant("GLUT_KEY_F6", 6, ID_FINAL);
  add_integer_constant("GLUT_KEY_F7", 7, ID_FINAL);
  add_integer_constant("GLUT_KEY_F8", 8, ID_FINAL);
  add_integer_constant("GLUT_KEY_F9", 9, ID_FINAL);
  add_integer_constant("GLUT_KEY_HOME", 106, ID_FINAL);
  add_integer_constant("GLUT_KEY_INSERT", 108, ID_FINAL);
  add_integer_constant("GLUT_KEY_LEFT", 100, ID_FINAL);
  add_integer_constant("GLUT_KEY_PAGE_DOWN", 105, ID_FINAL);
  add_integer_constant("GLUT_KEY_PAGE_UP", 104, ID_FINAL);
  add_integer_constant("GLUT_KEY_REPEAT_DEFAULT", 2, ID_FINAL);
  add_integer_constant("GLUT_KEY_REPEAT_OFF", 0, ID_FINAL);
  add_integer_constant("GLUT_KEY_REPEAT_ON", 1, ID_FINAL);
  add_integer_constant("GLUT_KEY_RIGHT", 102, ID_FINAL);
  add_integer_constant("GLUT_KEY_UP", 101, ID_FINAL);
  add_integer_constant("GLUT_LAYER_IN_USE", 801, ID_FINAL);
  add_integer_constant("GLUT_LEFT", 0, ID_FINAL);
  add_integer_constant("GLUT_LEFT_BUTTON", 0, ID_FINAL);
  add_integer_constant("GLUT_LUMINANCE", 512, ID_FINAL);
  add_integer_constant("GLUT_MENU_IN_USE", 1, ID_FINAL);
  add_integer_constant("GLUT_MENU_NOT_IN_USE", 0, ID_FINAL);
  add_integer_constant("GLUT_MENU_NUM_ITEMS", 300, ID_FINAL);
  add_integer_constant("GLUT_MIDDLE_BUTTON", 1, ID_FINAL);
  add_integer_constant("GLUT_MULTISAMPLE", 128, ID_FINAL);
  add_integer_constant("GLUT_NORMAL", 0, ID_FINAL);
  add_integer_constant("GLUT_NORMAL_DAMAGED", 804, ID_FINAL);
  add_integer_constant("GLUT_NOT_VISIBLE", 0, ID_FINAL);
  add_integer_constant("GLUT_NUM_BUTTON_BOX_BUTTONS", 607, ID_FINAL);
  add_integer_constant("GLUT_NUM_DIALS", 608, ID_FINAL);
  add_integer_constant("GLUT_NUM_MOUSE_BUTTONS", 605, ID_FINAL);
  add_integer_constant("GLUT_NUM_SPACEBALL_BUTTONS", 606, ID_FINAL);
  add_integer_constant("GLUT_NUM_TABLET_BUTTONS", 609, ID_FINAL);
  add_integer_constant("GLUT_OVERLAY", 1, ID_FINAL);
  add_integer_constant("GLUT_OVERLAY_DAMAGED", 805, ID_FINAL);
  add_integer_constant("GLUT_OVERLAY_POSSIBLE", 800, ID_FINAL);
  add_integer_constant("GLUT_OWNS_JOYSTICK", 613, ID_FINAL);
  add_integer_constant("GLUT_PARTIALLY_RETAINED", 2, ID_FINAL);
  add_integer_constant("GLUT_RED", 0, ID_FINAL);
  add_integer_constant("GLUT_RGB", 0, ID_FINAL);
  add_integer_constant("GLUT_RGBA", 0, ID_FINAL);
  add_integer_constant("GLUT_RIGHT_BUTTON", 2, ID_FINAL);
  add_integer_constant("GLUT_SCREEN_HEIGHT", 201, ID_FINAL);
  add_integer_constant("GLUT_SCREEN_HEIGHT_MM", 203, ID_FINAL);
  add_integer_constant("GLUT_SCREEN_WIDTH", 200, ID_FINAL);
  add_integer_constant("GLUT_SCREEN_WIDTH_MM", 202, ID_FINAL);
  add_integer_constant("GLUT_SINGLE", 0, ID_FINAL);
  add_integer_constant("GLUT_STENCIL", 32, ID_FINAL);
  add_integer_constant("GLUT_STEREO", 256, ID_FINAL);
  add_integer_constant("GLUT_TRANSPARENT_INDEX", 803, ID_FINAL);
  add_integer_constant("GLUT_UP", 1, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_HEIGHT", 909, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_HEIGHT_DELTA", 905, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_IN_USE", 901, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_POSSIBLE", 900, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_WIDTH", 908, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_WIDTH_DELTA", 904, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_X", 906, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_X_DELTA", 902, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_Y", 907, ID_FINAL);
  add_integer_constant("GLUT_VIDEO_RESIZE_Y_DELTA", 903, ID_FINAL);
  add_integer_constant("GLUT_VISIBLE", 1, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_ACCUM_ALPHA_SIZE", 114, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_ACCUM_BLUE_SIZE", 113, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_ACCUM_GREEN_SIZE", 112, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_ACCUM_RED_SIZE", 111, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_ALPHA_SIZE", 110, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_BLUE_SIZE", 109, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_BUFFER_SIZE", 104, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_COLORMAP_SIZE", 119, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_CURSOR", 122, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_DEPTH_SIZE", 106, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_DOUBLEBUFFER", 115, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_FORMAT_ID", 123, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_GREEN_SIZE", 108, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_HEIGHT", 103, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_NUM_CHILDREN", 118, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_NUM_SAMPLES", 120, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_PARENT", 117, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_RED_SIZE", 107, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_RGBA", 116, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_STENCIL_SIZE", 105, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_STEREO", 121, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_WIDTH", 102, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_X", 100, ID_FINAL);
  add_integer_constant("GLUT_WINDOW_Y", 101, ID_FINAL);
  post_init();
}

